@@ -1,6 +1,4 @@
-/**************************************
-** Created by Kevin from config.h.in **
-***************************************/
+/* include/llvm/Config/config.h.cmake corresponding to config.h.in. */
 
 #ifndef CONFIG_H
 #define CONFIG_H
@@ -14,15 +12,24 @@
 /* Directories clang will search for headers */
 #define C_INCLUDE_DIRS ""
 
-/* Define if CBE is enabled for printf %a output */
-#define ENABLE_CBE_PRINTF_A 1
+/* Default <path> to all compiler invocations for --sysroot=<path>. */
+#undef DEFAULT_SYSROOT
+
+/* Define if you want backtraces on crash */
+#define ENABLE_BACKTRACES
 
 /* Define if position independent code is enabled */
 #define ENABLE_PIC
 
-/* Define if timestamp information (e.g., __DATE___) is allowed */
+/* Define if timestamp information (e.g., __DATE__) is allowed */
 #define ENABLE_TIMESTAMPS 1
 
+/* Directory where gcc is installed. */
+#undef GCC_INSTALL_PREFIX
+
+/* Define to 1 if you have the `arc4random' function. */
+#define HAVE_ARC4RANDOM
+
 /* Define to 1 if you have the `argz_append' function. */
 /* #undef HAVE_ARGZ_APPEND */
 
@@ -45,7 +52,7 @@
 #define HAVE_ASSERT_H 1
 
 /* Define to 1 if you have the `backtrace' function. */
-#undef HAVE_BACKTRACE
+/* #undef HAVE_BACKTRACE */
 
 /* Define to 1 if you have the `bcopy' function. */
 #undef HAVE_BCOPY
@@ -54,7 +61,7 @@
 #define HAVE_CEILF 1
 
 /* Define if the neat program is available */
-/* #undef HAVE_CIRCO */
+#define HAVE_CIRCO 1
 
 /* Define to 1 if you have the `closedir' function. */
 #define HAVE_CLOSEDIR 1
@@ -95,10 +102,10 @@
 /* #undef HAVE_DL_H */
 
 /* Define if the dot program is available */
-/* #undef HAVE_DOT */
+#define HAVE_DOT 1
 
 /* Define if the dotty program is available */
-/* #undef HAVE_DOTTY */
+#define HAVE_DOTTY 1
 
 /* Define if you have the _dyld_func_lookup function. */
 #undef HAVE_DYLD
@@ -116,7 +123,7 @@
 #define HAVE_FCNTL_H 1
 
 /* Define if the neat program is available */
-/* #undef HAVE_FDP */
+#define HAVE_FDP 1
 
 /* Define to 1 if you have the <fenv.h> header file. */
 #define HAVE_FENV_H 1
@@ -155,7 +162,7 @@
 #define HAVE_GETTIMEOFDAY 1
 
 /* Define if the Graphviz program is available */
-#undef HAVE_GRAPHVIZ
+/* #undef HAVE_GRAPHVIZ */
 
 /* Define if the gv program is available */
 /* #undef HAVE_GV */
@@ -271,7 +278,7 @@
 #define HAVE_NEARBYINTF 1
 
 /* Define if the neat program is available */
-/* #undef HAVE_NEATO */
+#define HAVE_NEATO 1
 
 /* Define to 1 if you have the `opendir' function. */
 #define HAVE_OPENDIR 1
@@ -444,7 +451,7 @@
 #define HAVE_TERMIOS_H 1
 
 /* Define if the neat program is available */
-/* #undef HAVE_TWOPI */
+#define HAVE_TWOPI 1
 
 /* Define to 1 if the system has the type `uint64_t'. */
 #define HAVE_UINT64_T 1
@@ -534,7 +541,7 @@
 /* #undef LLVM_DATADIR */
 
 /* Target triple LLVM will generate code for by default */
-#define LLVM_DEFAULT_TARGET_TRIPLE "sparc64-unknown-openbsd5.2"
+#define LLVM_DEFAULT_TARGET_TRIPLE "powerpc-unknown-openbsd5.3"
 
 /* Installation directory for documentation */
 /* #undef LLVM_DOCSDIR */
@@ -548,6 +555,9 @@
 /* Has gcc/MSVC atomic intrinsics */
 #define LLVM_HAS_ATOMICS 1
 
+/* Host triple LLVM will be executed on */
+#define LLVM_HOSTTRIPLE "powerpc-unknown-openbsd5.3"
+
 /* Installation directory for include files */
 /* #undef LLVM_INCLUDEDIR */
 
@@ -561,25 +571,25 @@
 /* #undef LLVM_MANDIR */
 
 /* LLVM architecture name for the native architecture, if available */
-#define LLVM_NATIVE_ARCH Sparc
+#define LLVM_NATIVE_ARCH PowerPC
 
 /* LLVM name for the native AsmParser init function, if available */
 /* #undef LLVM_NATIVE_ASMPARSER */
 
 /* LLVM name for the native AsmPrinter init function, if available */
-#define LLVM_NATIVE_ASMPRINTER LLVMInitializeSparcAsmPrinter
+#define LLVM_NATIVE_ASMPRINTER LLVMInitializePowerPCAsmPrinter
 
 /* LLVM name for the native Disassembler init function, if available */
 /* #undef LLVM_NATIVE_DISASSEMBLER */
 
 /* LLVM name for the native Target init function, if available */
-#define LLVM_NATIVE_TARGET LLVMInitializeSparcTarget
+#define LLVM_NATIVE_TARGET LLVMInitializePowerPCTarget
 
 /* LLVM name for the native TargetInfo init function, if available */
-#define LLVM_NATIVE_TARGETINFO LLVMInitializeSparcTargetInfo
+#define LLVM_NATIVE_TARGETINFO LLVMInitializePowerPCTargetInfo
 
 /* LLVM name for the native target MC init function, if available */
-#define LLVM_NATIVE_TARGETMC LLVMInitializeSparcTargetMC
+#define LLVM_NATIVE_TARGETMC LLVMInitializePowerPCTargetMC
 
 /* Define if this is Unixish platform */
 #define LLVM_ON_UNIX 1
@@ -588,16 +598,16 @@
 /* #undef LLVM_ON_WIN32 */
 
 /* Define to path to circo program if found or 'echo circo' otherwise */
-/* #undef LLVM_PATH_CIRCO */
+#define LLVM_PATH_CIRCO "/usr/local/bin/circo"
 
 /* Define to path to dot program if found or 'echo dot' otherwise */
-/* #undef LLVM_PATH_DOT */
+#define LLVM_PATH_DOT "/usr/local/bin/dot"
 
 /* Define to path to dotty program if found or 'echo dotty' otherwise */
-/* #undef LLVM_PATH_DOTTY */
+#define LLVM_PATH_DOTTY "/usr/local/bin/dotty"
 
 /* Define to path to fdp program if found or 'echo fdp' otherwise */
-/* #undef LLVM_PATH_FDP */
+#define LLVM_PATH_FDP "/usr/local/bin/fdp"
 
 /* Define to path to Graphviz program if found or 'echo Graphviz' otherwise */
 /* #undef LLVM_PATH_GRAPHVIZ */
@@ -606,10 +616,10 @@
 /* #undef LLVM_PATH_GV */
 
 /* Define to path to neato program if found or 'echo neato' otherwise */
-/* #undef LLVM_PATH_NEATO */
+#define LLVM_PATH_NEATO "/usr/local/bin/neato"
 
 /* Define to path to twopi program if found or 'echo twopi' otherwise */
-/* #undef LLVM_PATH_TWOPI */
+#define LLVM_PATH_TWOPI "/usr/local/bin/twopi"
 
 /* Define to path to xdot.py program if found or 'echo xdot.py' otherwise */
 /* #undef LLVM_PATH_XDOT_PY */
@@ -617,11 +627,17 @@
 /* Installation prefix directory */
 #define LLVM_PREFIX "/usr/local"
 
+/* Define if we have the Intel JIT API runtime support library */
+/* #undef LLVM_USE_INTEL_JITEVENTS */
+
+/* Define if we have the oprofile JIT-support library */
+/* #undef LLVM_USE_OPROFILE */
+
 /* Major version of the LLVM API */
 #define LLVM_VERSION_MAJOR 3
 
 /* Minor version of the LLVM API */
-#define LLVM_VERSION_MINOR 1
+#define LLVM_VERSION_MINOR 2
 
 /* Define if the OS needs help to load dependent libraries for dlopen(). */
 /* #undef LTDL_DLOPEN_DEPLIBS */
@@ -654,13 +670,13 @@
 #define PACKAGE_NAME "LLVM"
 
 /* Define to the full name and version of this package. */
-#define PACKAGE_STRING "LLVM 3.1svn"
+#define PACKAGE_STRING "LLVM 3.2"
 
 /* Define to the one symbol short name of this package. */
 #undef PACKAGE_TARNAME
 
 /* Define to the version of this package. */
-#define PACKAGE_VERSION "3.1svn"
+#define PACKAGE_VERSION "3.2"
 
 /* Define as the return type of signal handlers (`int' or `void'). */
 #define RETSIGTYPE void
@@ -713,10 +729,4 @@
 /* Added by Kevin -- Maximum path length */
 #define MAXPATHLEN 2024
 
-/* Support for Intel JIT Events API is enabled */
-/* #undef LLVM_USE_INTEL_JITEVENTS */
-
-/* Support for OProfile JIT API is enabled */
-/* #undef LLVM_USE_OPROFILE */
-
 #endif
